@@ -8,6 +8,7 @@ from sql.aggregate import Sum
 from sql.conditionals import Coalesce
 
 from trytond import backend
+from trytond.i18n import gettext
 from trytond.model import (ModelView, ModelSQL, DeactivableMixin, fields,
     Unique, tree)
 from trytond.wizard import Wizard, StateView, StateAction, Button
@@ -15,6 +16,8 @@ from trytond.pyson import Eval, If, PYSONEncoder, PYSONDecoder
 from trytond.transaction import Transaction
 from trytond.pool import Pool
 
+from .exceptions import AccountValidationError
+
 __all__ = ['Account', 'AccountDistribution',
     'OpenChartAccountStart', 'OpenChartAccount',
     'AnalyticAccountEntry', 'AnalyticMixin']
@@ -83,15 +86,6 @@ class Account(
         ('closed', 'Closed'),
         ], 'State', required=True)
     note = fields.Text('Note')
-    display_balance = fields.Selection([
-        ('debit-credit', 'Debit - Credit'),
-        ('credit-debit', 'Credit - Debit'),
-        ], 'Display Balance', required=True)
-    mandatory = fields.Boolean('Mandatory', states={
-            'invisible': Eval('type') != 'root',
-            },
-        depends=['type'],
-        help="Make this account mandatory when filling documents")
     distributions = fields.One2Many(
         'analytic_account.account.distribution', 'parent',
         "Distributions",
@@ -109,11 +103,6 @@ class Account(
         super(Account, cls).__setup__()
         cls._order.insert(0, ('code', 'ASC'))
         cls._order.insert(1, ('name', 'ASC'))
-        cls._error_messages.update({
-                'invalid_distribution': (
-                    'The distribution sum of account "%(account)s" '
-                    'is not 100%%.'),
-                })
 
     @classmethod
     def __register__(cls, module_name):
@@ -123,6 +112,9 @@ class Account(
         # Migration from 4.0: remove currency
         table.not_null_action('currency', action='remove')
 
+        # Migration from 5.0: remove display_balance
+        table.drop_column('display_balance')
+
     @staticmethod
     def default_company():
         return Transaction().context.get('company')
@@ -135,14 +127,6 @@ class Account(
     def default_state():
         return 'draft'
 
-    @staticmethod
-    def default_display_balance():
-        return 'credit-debit'
-
-    @staticmethod
-    def default_mandatory():
-        return False
-
     @classmethod
     def validate(cls, accounts):
         super(Account, cls).validate(accounts)
@@ -153,9 +137,9 @@ class Account(
         if self.type != 'distribution':
             return
         if sum((d.ratio for d in self.distributions)) != 1:
-            self.raise_user_error('invalid_distribution', {
-                    'account': self.rec_name,
-                    })
+            raise AccountValidationError(
+                gettext('analytic_account.msg_invalid_distribution',
+                    account=self.rec_name))
 
     @fields.depends('company')
     def on_change_with_currency(self, name=None):
@@ -169,10 +153,9 @@ class Account(
         return 2
 
     @fields.depends('parent', 'type',
-        '_parent_parent.id', '_parent_parent.root', '_parent_parent.type')
+        '_parent_parent.root', '_parent_parent.type')
     def on_change_parent(self):
-        if (self.parent and self.parent.id is not None and self.parent.id > 0
-                and self.type != 'root'):
+        if self.parent and self.type != 'root':
             if self.parent.type == 'root':
                 self.root = self.parent
             else:
@@ -205,7 +188,7 @@ class Account(
                 ).join(move_line, 'LEFT',
                 condition=move_line.id == line.move_line
                 ).select(table.id,
-                Sum(Coalesce(line.debit, 0) - Coalesce(line.credit, 0)),
+                Sum(Coalesce(line.credit, 0) - Coalesce(line.debit, 0)),
                 where=(table.type != 'view')
                 & table.id.in_(all_ids)
                 & (table.active == True) & line_query,
@@ -226,8 +209,6 @@ class Account(
                     ])
             for child in childs:
                 balance += account_sum[child.id]
-            if account.display_balance == 'credit-debit' and balance:
-                balance *= -1
             exp = Decimal(str(10.0 ** -account.currency_digits))
             balances[account.id] = balance.quantize(exp)
         return balances
@@ -313,12 +294,11 @@ class Account(
             if remainder:
                 i = 0
                 while remainder:
-                    account, current_amount = result[i]
+                    account, amount = result[i]
                     rounding = self.currency.rounding.copy_sign(remainder)
-                    result[i] = (account, current_amount + rounding)
+                    result[i] = (account, amount - rounding)
                     remainder -= rounding
                     i = (i + 1) % len(result)
-            assert sum(a for _, a in result) == amount
             return result
 
 
@@ -397,16 +377,11 @@ class AnalyticAccountEntry(ModelView, ModelSQL):
         depends=['company'])
     account = fields.Many2One('analytic_account.account', 'Account',
         ondelete='RESTRICT',
-        states={
-            'required': Eval('required', False),
-            },
         domain=[
             ('root', '=', Eval('root')),
             ('type', 'in', ['normal', 'distribution']),
             ],
-        depends=['root', 'required', 'company'])
-    required = fields.Function(fields.Boolean('Required'),
-        'on_change_with_required')
+        depends=['root', 'company'])
     company = fields.Function(fields.Many2One('company.company', 'Company'),
         'on_change_with_company', searcher='search_company')
 
@@ -449,7 +424,7 @@ class AnalyticAccountEntry(ModelView, ModelSQL):
         t = cls.__table__()
         cls._sql_constraints += [
             ('root_origin_uniq', Unique(t, t.origin, t.root),
-                'Only one account is allowed per analytic root and origin.'),
+                'analytic_account.msg_root_origin_unique'),
             ]
 
     @classmethod
@@ -465,12 +440,6 @@ class AnalyticAccountEntry(ModelView, ModelSQL):
                 ])
         return [(None, '')] + [(m.model, m.name) for m in models]
 
-    @fields.depends('root')
-    def on_change_with_required(self, name=None):
-        if self.root:
-            return self.root.mandatory
-        return False
-
     def on_change_with_company(self, name=None):
         return None
 
@@ -504,14 +473,6 @@ class AnalyticMixin(object):
     analytic_accounts_size = fields.Function(fields.Integer(
             'Analytic Accounts Size'), 'get_analytic_accounts_size')
 
-    @classmethod
-    def __setup__(cls):
-        super(AnalyticMixin, cls).__setup__()
-        cls._error_messages.update({
-                'root_account': ('Some mandatory root account are missing '
-                    'on "%(name)s"'),
-                })
-
     @classmethod
     def __register__(cls, module_name):
         pool = Pool()
@@ -553,7 +514,6 @@ class AnalyticMixin(object):
                 ])
         for account in root_accounts:
             accounts.append({
-                    'required': account.mandatory,
                     'root': account.id,
                     })
         return accounts
@@ -571,29 +531,3 @@ class AnalyticMixin(object):
     def get_analytic_accounts_size(cls, records, name):
         roots = cls.default_analytic_accounts_size()
         return {r.id: roots for r in records}
-
-    @classmethod
-    def validate(cls, analytics):
-        super(AnalyticMixin, cls).validate(analytics)
-        cls.check_roots(analytics)
-
-    @classmethod
-    def check_roots(cls, analytics):
-        "Check that all mandatory root entries are defined in entries"
-        pool = Pool()
-        Account = pool.get('analytic_account.account')
-        all_mandatory_roots = {a for a in Account.search([
-                ('type', '=', 'root'),
-                ('mandatory', '=', True),
-                ])}
-        for analytic in analytics:
-            analytic_roots = {e.root for e in analytic.analytic_accounts}
-            companies = {e.company for e in analytic.analytic_accounts}
-            mandatory_roots = set()
-            for mandatory in all_mandatory_roots:
-                if mandatory.company in companies:
-                    mandatory_roots.add(mandatory)
-            if not mandatory_roots <= analytic_roots:
-                cls.raise_user_error('root_account', {
-                        'name': analytic.rec_name,
-                        })
